package com.junmp.jyzb.controller;

import com.junmp.jyzb.api.bean.dto.*;
import com.junmp.jyzb.api.bean.query.*;
import com.junmp.jyzb.api.bean.req.EquipmentBindReq;
import com.junmp.jyzb.entity.InventorySummary;
import com.junmp.jyzb.service.*;
import com.junmp.v2.common.bean.request.ValidationApi;
import com.junmp.v2.common.bean.response.ApiRes;
import com.junmp.v2.db.api.page.PageResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@Slf4j
@RequestMapping("/Inventory")
@Api(tags = "库存模块")
public class InventoryController {
    @Resource
    public InventoryService inventoryService;
    @Resource
    public InventorySummaryService inventorySummaryService;
    @Resource
    public MQProductService mqProductService;


    @GetMapping("/testMQ")
    @ApiOperation("根据组织机构查询库存信息")
    public ApiRes<Boolean> testMQ( ) {
        String aa="";
        return ApiRes.success(mqProductService.sendMessage());
    }


    //填充汇总信息(将表中的数据base_inventory_summary进行删除并且根据base_inventory表中数据同步)
    @PostMapping("/SetInventoryMsg")
    @ApiOperation("填充汇总表信息")
    public ApiRes<Boolean> setInventoryMsg() {
        boolean result = inventoryService.setInventoryMsg();
        if (!result){
            return ApiRes.failure("汇总失败");
        }
        return ApiRes.success(result);
    }


    //装备绑定箱门
    @PostMapping("/EquipmentBind")
    @ApiOperation("装备绑定箱门")
    public ApiRes<Boolean> InventoryBind(@RequestBody @Validated(ValidationApi.edit.class) EquipmentBindReq req){
        return ApiRes.success(inventoryService.InventoryBind(req));
    }

    //手动修改仓库数量信息，修改的是base_warehouse中的个别字段（总数，总价,入库数，出库数），是通过inventory_summary进行统计的
    @PostMapping("/UpdateWarehouseInsNum")
    @ApiOperation("手动重置仓库库存数量信息")
    public ApiRes<Boolean> UpdateWarehouseInsNum(@RequestBody @Validated(ValidationApi.detail.class) WarehouseReq req){
        return ApiRes.success(inventoryService.UpdateWarehouseInsNum(req));
    }

    //手动修改单警柜数量信息，修改的是base_cabinet中的个别字段（总数，总价,入库数，出库数），是通过inventory_summary进行统计的
    @PostMapping("/UpdateCabinetInsNum")
    @ApiOperation("手动重置单警柜库存数量信息")
    public ApiRes<Boolean> UpdateCabinetInsNum(@RequestBody @Validated(ValidationApi.detail.class)CabinetReq req){
        return ApiRes.success(inventoryService.UpdateCabinetInsNum(req));
    }

    //根据Epc获取装备简要信息
    @PostMapping("/GetInvInfoByEpc")
    @ApiOperation("根据EPC列表获取装备信息（简要）")
    public ApiRes<List<InventoryDto>> GetInvInfoByEpc(@RequestBody InventoryReq req){
        return ApiRes.success(inventoryService.GetInvInfoByEpc(req));
    }


    //根据组织机构id/单警柜id/仓库id查询出库存的汇总信息√
    @PostMapping("/GetEquipmentInfo")
    @ApiOperation("根据条件查询库存汇总信息")
    public ApiRes<PageResult<InventorySummary>> getEquipmentInfo(@RequestBody InventorySumReq req) {
        return ApiRes.success(inventorySummaryService.getEquipmentInfo(req));
    }

    //根据组织机构/单警柜id/仓库id查询每件装备细信的详息√
    @PostMapping("/EqsBySearchType")
    @ApiOperation("查询库存装备详细信息")
    public ApiRes<List<InventoryDto>> EqsBySearchType(@RequestBody @Validated(ValidationApi.edit.class) InventoryReq req ){
        return ApiRes.success(inventoryService.EqsBySearchType(req));
    }


    //根据装备id查询出装备的详细信息
    @PostMapping("/GetDetail")
    @ApiOperation("根据装备id查询装备明细信息")
    public ApiRes<InventoryDto> getInventoryDetail(@RequestBody @Validated(ValidationApi.detail.class) InventoryReq req) {
        return  ApiRes.success(inventoryService.getInventoryDetail(req));
    }

    //查询箱门（cabinetBox下）所有装备简要信息列表（传递cabinetBoxId）
    @PostMapping("/searchEqsByBoxId")
    @ApiOperation("查询某一箱门下的所有装备简要信息")
    public ApiRes<List<EqsBriefDto>> searchEqsByBoxId(@RequestBody @Validated(ValidationApi.detail.class) CabinetBoxReq req){
        return ApiRes.success(inventoryService.searchEqsByBoxId(req));
    }

    //查询单警柜箱门下的汇总信息（传递cabinetBoxId）
    @PostMapping("/EqsByCabinetBoxId")
    @ApiOperation("查询箱门下的汇总信息")
    public ApiRes<List<InventorySummary>> EqsByCabinetId(@RequestBody @Validated(ValidationApi.detail.class)CabinetBoxReq req){
        return ApiRes.success(inventoryService.EqsByCabinetBoxId(req));
    }


    @PostMapping("/ShelfInventoryList")
    @ApiOperation("根据货架id查询装备信息汇总")
    public ApiRes<List<InventorySummary>> ShelfInventoryList(@RequestBody @Validated(ValidationApi.edit.class) ShelfReq req){
        return ApiRes.success(inventoryService.ShelfInventoryList(req));
    }

    @PostMapping("/ShelfInventoryDetail")
    @ApiOperation(("/通过货架id查询每件装备信息简要"))
    public ApiRes<List<EqsBriefDto>> AreaInventoryDetail(@RequestBody @Validated(ValidationApi.edit.class) ShelfReq req){
        return ApiRes.success(inventoryService.ShelfInventoryDetail(req));
    }

    @PostMapping("/AreaInventoryList")
    @ApiOperation("根据区域id查询装备信息汇总")
    public ApiRes<List<InventorySummary>> AreaInventoryList(@RequestBody @Validated(ValidationApi.edit.class) WarehouseAreaReq req){
        return ApiRes.success(inventoryService.AreaInventoryList(req));
    }

    //导出,通过传递组织机构id，将该组织机构下的所有装备信息进行导出
    @PostMapping("/ExportInventoryExcel")
    @ApiOperation("组织机构装备导出")
    public void ExportInventoryExcel(@RequestBody @Validated(ValidationApi.export.class) InventoryReq req) {
        inventoryService.ExportInventoryExcel(req);
    }

    //装备出入库记录（通过装备id）
    @PostMapping("/GetInOutRecords")
    @ApiOperation("装备出入库记录")
    public ApiRes<List<InOutRecordsDto>> getInOutRecords(@RequestBody @Validated(ValidationApi.detail.class) InventoryReq req){
        return ApiRes.success(inventoryService.getInOutRecords(req));
    }

    //根据条件（组织机构，装备类型id，装备型号id，装备型号名称）查询装备总数，在库数，出库数，单警柜在库数，单警柜出库数，报废数
    //查询装备数量报表
    @PostMapping("/GetListEquipment")
    @ApiOperation("根据条件查询数量汇总")
    public ApiRes<List<EqsSumDto>> GetListEquipment(@RequestBody InventoryReq req){
        return ApiRes.success(inventoryService.GetListEquipment(req));
    }


}
