package com.junmp.jyzb.controller;

import cn.hutool.core.util.ObjectUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.junmp.jyzb.api.bean.dto.LogSummaryDto;
import com.junmp.jyzb.api.bean.dto.OrderDetailDto;
import com.junmp.jyzb.api.bean.dto.OrderDto;
import com.junmp.jyzb.api.bean.dto.OrderMainDto;
import com.junmp.jyzb.api.bean.query.OrderDetailReq;
import com.junmp.jyzb.api.bean.query.OrderMainReq;
import com.junmp.jyzb.api.bean.req.UpdateOrderReq;
import com.junmp.jyzb.service.LogSummaryService;
import com.junmp.jyzb.service.OrderLogService;
import com.junmp.jyzb.service.OrderService;
import com.junmp.v2.common.bean.request.ValidationApi;
import com.junmp.v2.common.bean.response.ApiRes;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
@Slf4j
@RequestMapping("/Order")
@Api(tags = "业务模块")
public class OrderController {
    @Resource
    public OrderService orderService;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Resource
    private LogSummaryService logSummaryService;

    @Resource
    private OrderLogService orderLogService;

    //通过工作流进行操作处理的
    @PostMapping("/AddOrder")
    @ApiOperation("新增任务单")
    public ApiRes<String> addOrder(@RequestBody UpdateOrderReq req) {
        //判断是否有工作流id，如果有则将状态进行修改
        if (ObjectUtil.isEmpty(req.getProcessId()))
        {
            req.setExamineState("none");
        }
        else
        {
            req.setExamineState("working");
        }
        return ApiRes.success(orderService.AddOrder(req));
    }


    //内部处理，任务单状态直接设置成finished 添加到数据并且直接推送到消息队列中去
    @PostMapping("/AddFinishOrder")
    @ApiOperation("新增已完成的任务单")
    public ApiRes<String> AddFinishOrder(@RequestBody UpdateOrderReq req)  {
        req.setExamineState("finished");
        return ApiRes.success(orderService.AddFinishOrder(req));
    }


    //修改任务单只允许工作流id为空的时候（视为草稿可以进行修改）
    @PostMapping("/UpdateOrder")
    @ApiOperation("修改任务单")
    public ApiRes<Boolean>  updateOrder(@RequestBody @Validated(ValidationApi.edit.class) UpdateOrderReq req) {
        return ApiRes.success(orderService.updateOrder(req));
    }

    //根据任务单id查询对应的任务单信息
    @PostMapping("/GetOrderMain")
    @ApiOperation("查询任务单信息")
    public ApiRes<OrderMainDto> getOrderMain(@RequestBody @Validated(ValidationApi.detail.class)OrderMainReq req) {
        OrderMainDto orderMainDto = orderService.showOrderMain(req);
        return ApiRes.success(orderMainDto);
    }
    //根据任务单id查询详细信息
    @PostMapping("/GetOrderDetail")
    @ApiOperation("根据任务单id查看业务明细")
    public ApiRes<List<OrderDetailDto>> getOrderDetail(@RequestBody @Validated(ValidationApi.detail.class) OrderDetailReq req){
        List<OrderDetailDto> orderDetail = orderService.getOrderDetail(req);
        return ApiRes.success(orderDetail);
    }

//    //将工作单添加到rabbitmq中去
//    @PostMapping("/AddOrderRabbitMq")
//    @ApiOperation("将工作单添加到rabbitmq消息队列中去")
//    public ApiRes<Boolean> AddOrderRabbitMq(){
//        List<OrderDto> order = orderService.getOrder();
//        List<String> orderId=new ArrayList<>();
//        for (OrderDto orderDto:order) {
//            //将routingKey设置为收物机构id
//            rabbitTemplate.convertAndSend("OrderExchange",orderDto.getEndOrgId(),orderDto);
//            orderId.add(orderDto.getId());
//        }
//        //将未被上传的数据进行上传到rabbitmq中，同时添加到数据库，标记为已上传数据，比避免同意数据多次或者重复上传
//        boolean b = orderLogService.addOrder(orderId);
//        return ApiRes.success(true);
//    }



}
