package com.junmp.jyzb.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.junmp.jyzb.api.bean.dto.*;
import com.junmp.jyzb.api.bean.query.*;
import com.junmp.jyzb.api.bean.req.EquipmentBindReq;
import com.junmp.jyzb.api.bean.vo.InventoryVo;
import com.junmp.jyzb.api.exception.enums.CabinetBoxExceptionEnum;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.api.exception.enums.InventoryExceptionEnum;
import com.junmp.jyzb.entity.*;

import com.junmp.jyzb.service.*;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import com.junmp.v2.common.util.HttpServletUtil;
import com.junmp.v2.office.api.OfficeExcelApi;
import com.junmp.v2.office.api.bean.ExcelExportParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import com.junmp.jyzb.mapper.InventoryMapper;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;


@Service
@Slf4j
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {
    @Resource
    private InventoryMapper inventoryMapper;
    @Resource
    private EquipmentTypeService equipmentTypeService;
    @Resource
    private EquipmentSizeService equipmentSizeService;
    @Resource
    private WarehouseService warehouseService;
    @Resource
    private PubOrgService PubOrgService;

    @Resource
    private CabinetBoxService cabinetBoxService;

    @Resource
    private PubOrgService pubOrgService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private SupplierService supplierService;
    @Resource
    private ShelfService shelfService;
    @Resource
    private WarehouseAreaService warehouseAreaService;

    @Resource
    private LogSummaryService logSummaryService;
    @Resource
    private LogDetailService logDetailService;

    @Resource
    private OfficeExcelApi officeExcelApi;

    @Override
    public InventoryDto getInventoryDetail(InventoryReq req) {
        Inventory inventory = InventoryExist(req.getId());
        InventoryDto bean = BeanUtil.toBean(inventory, InventoryDto.class);
        // 查询与库存表关联的其他数据
        String orgName= PubOrgService.getById(inventory.getOrgId()).getOrgName();
        String TypeName = equipmentTypeService.getById(inventory.getTypeId()).getName();
        String SizeName = equipmentSizeService.getById(inventory.getSizeId()).getName();
        // 创建并填充对象
        bean.setSizeName(SizeName);
        bean.setTypeName(TypeName);
        bean.setOrgName(orgName);
        return bean;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setInventoryMsg(){
        //清空数据库中已存在的信息
        inventoryMapper.deleteAllMsg();
        //放入单警柜信息汇总
        boolean b = inventoryMapper.SetCabinetInventory();
        //放入仓库区域信息汇总
        boolean b1 = inventoryMapper.SetWarehouseInventory();
        if (b && b1){
            return true;
        }else{
            return false;
        }
    }



    //设置仓库库存数量（总数，在库数，出库数，价格）
    @Override
    public boolean UpdateWarehouseInsNum(WarehouseReq req) {
        Warehouse warehouse = warehouseService.WarehoustExist(req.getId());
        boolean b = warehouseService.SetInventoryInfo(warehouse.getId());
        return b;
    }

    //设置单警柜库存数量（总数，在库数，出库数，价格）
    @Override
    public boolean UpdateCabinetInsNum(CabinetReq req) {
        Cabinet cabinet = cabinetService.CabinetExist(req.getId());
        boolean b = cabinetService.SetInventoryInfo(cabinet.getId());
        return b;
    }

    //装备绑定箱门
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean InventoryBind(EquipmentBindReq req) {
        //判断箱门是否存在
        CabinetBox cabinetBox = cabinetBoxService.CabinetBoxExist(req.getCabinetBoxId());
        if (Objects.isNull(cabinetBox)){
            throw new ServiceException(CabinetBoxExceptionEnum.CABINETBOX_ISNOT_EXISTS);
        }
        List<String> equipmentList = req.getEquipmentList();
        if (equipmentList.size()==0){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        List<Inventory> inventoryList=new ArrayList<>();
        for (String id: equipmentList) {
            Inventory inventory = getById(id);
            //locationId:根据单警柜的不同存放的内容不同，单警柜存放箱门id，仓库存放仓库id
            inventory.setLocationId(req.getCabinetBoxId());
            inventoryList.add(inventory);
        }
        return updateBatchById(inventoryList);
    }

    //根据Epc查询装备信息(简要信息)
    @Override
    public List<InventoryDto> GetInvInfoByEpc(InventoryReq req) {
        List<InventoryDto> inventoryDtoList=new ArrayList<>();
        //判断传入的是否为空
        List<String> epcList = req.getEpcList();
        if (epcList.size()==0){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        //迭代器遍历列表
        Iterator<String> iterator = epcList.iterator();
        while (iterator.hasNext()){
            String epc = iterator.next();
            Inventory inventory = getOne(new LambdaQueryWrapper<Inventory>().eq(Inventory::getEpc, epc));
            if (Objects.isNull(inventory)){
                throw new ServiceException(InventoryExceptionEnum.INVENTORY_NOT_EXIST);
            }
            InventoryDto inventoryDto = new InventoryDto();
            BeanPlusUtil.copyProperties(inventory, inventoryDto);
            //设置型号名称，类别名称，供应商名称
            String sizeName = equipmentSizeService.getById(inventory.getSizeId()).getName();
            String typeName = equipmentTypeService.getById(inventory.getTypeId()).getName();
            String supplierName = supplierService.getById(inventory.getSupplierId()).getName();
            inventoryDto.setSizeName(sizeName);
            inventoryDto.setTypeName(typeName);
            inventoryDto.setSupplierName(supplierName);
            inventoryDtoList.add(inventoryDto);
        }
        return inventoryDtoList;
    }

    @Override
    public List<InventoryDto> EqsBySearchType(InventoryReq req) {
        //判断传入的queryType参数是否是正确的
        if (!req.getQueryType().equals("org") && !req.getQueryType().equals("warehouse") && !req.getQueryType().equals("cabinet")){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        //判断如果传递了类别，判断传递的值是否为空或者是否不存在
        if (ObjectUtil.isNotNull(req.getOrgId())){
            pubOrgService.PubOrgExist(req.getOrgId());
        } else if (ObjectUtil.isNotNull(req.getWarehouseId())){
            warehouseService.WarehoustExist(req.getWarehouseId());
        } else if (ObjectUtil.isNotNull(req.getCabinetId())){
            cabinetService.CabinetExist(req.getCabinetId());
        }else {
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        return inventoryMapper.EqsBySearchType(req);
    }

    //查询箱门（cabinetBox下）所有装备简要信息列表（传递cabinetBoxId）
    @Override
    public List<EqsBriefDto> searchEqsByBoxId(CabinetBoxReq req) {
        List<Inventory> list = list(new LambdaQueryWrapper<Inventory>()
                .eq(Inventory::getLocationId, req.getId()));
        if (list.size()==0){
            throw new ServiceException(CabinetBoxExceptionEnum.CABINETBOX_ISNOT_EXISTS);
        }
        List<EqsBriefDto> eqsBriefDtoList=new ArrayList<>();
        Iterator<Inventory> iterator = list.iterator();
        while (iterator.hasNext()){
            Inventory next = iterator.next();
            EqsBriefDto eqsBriefDto = new EqsBriefDto();
            BeanPlusUtil.copyProperties(next,eqsBriefDto);
            //设置typeName
            eqsBriefDto.setTypeName(equipmentSizeService.getOne(new LambdaQueryWrapper<EquipmentSize>()
                    .eq(EquipmentSize::getId,eqsBriefDto.getSizeId())).getName());
            //设置sizeName
            eqsBriefDto.setSizeName(equipmentTypeService.getOne(new LambdaQueryWrapper<EquipmentType>()
                    .eq(EquipmentType::getId,eqsBriefDto.getTypeId())).getName());
            //设置supplierName
            eqsBriefDto.setSupplierName(supplierService.getOne(new LambdaQueryWrapper<Supplier>()
                    .eq(Supplier::getId,eqsBriefDto.getSupplierId())).getName());
            eqsBriefDtoList.add(eqsBriefDto);
        }
        return eqsBriefDtoList;
    }


    //根据货架id获取装备列表汇总信息
    @Override
    public List<InventorySummary> ShelfInventoryList(ShelfReq req) {
        //判断货架是否存在
        Shelf shelf = shelfService.ShelfExist(req.getShelfId());
        warehouseService.WarehoustExist(shelf.getWarehouseId());
        //通过货架id获取装备列表汇总
        return inventoryMapper.getSumByShelf(req.getShelfId());
    }

    //通过货架id查询装备简要信息
    @Override
    public List<EqsBriefDto> ShelfInventoryDetail(ShelfReq req) {
        //判断货架是否存在
        Shelf shelf = shelfService.ShelfExist(req.getShelfId());
        warehouseService.WarehoustExist(shelf.getWarehouseId());
        List<Inventory> list = list(new LambdaQueryWrapper<Inventory>()
                .eq(Inventory::getShelfId, req.getShelfId()));
        List<EqsBriefDto> eqsBriefDtoList=new ArrayList<>();
        for (Inventory inventory:list) {
            EqsBriefDto eqsBriefDto = new EqsBriefDto();
            BeanPlusUtil.copyProperties(inventory,eqsBriefDto);
            //设置typeName
            eqsBriefDto.setTypeName(equipmentSizeService.getOne(new LambdaQueryWrapper<EquipmentSize>()
                    .eq(EquipmentSize::getId,eqsBriefDto.getSizeId())).getName());
            //设置sizeName
            eqsBriefDto.setSizeName(equipmentTypeService.getOne(new LambdaQueryWrapper<EquipmentType>()
                    .eq(EquipmentType::getId,eqsBriefDto.getTypeId())).getName());
            //设置supplierName
            eqsBriefDto.setSupplierName(supplierService.getOne(new LambdaQueryWrapper<Supplier>()
                    .eq(Supplier::getId,eqsBriefDto.getSupplierId())).getName());
            eqsBriefDtoList.add(eqsBriefDto);
        }
        return eqsBriefDtoList;
    }

    /**
     * 通过区域id查询装备汇总信息
     * @param req
     * @return
     */
    @Override
    public List<InventorySummary> AreaInventoryList(WarehouseAreaReq req) {
        //判断区域是否存在
        WarehouseArea warehouseArea = warehouseAreaService.WarehouseAreaExist(req.getId());
        warehouseService.WarehoustExist(warehouseArea.getWarehouseId());
        //通过区域id获取装备列表汇总
        return inventoryMapper.getSumByArea(req.getId());
    }



    //通过单警柜箱门id获取箱门下的装备汇总信息
    @Override
    public List<InventorySummary> EqsByCabinetBoxId(CabinetBoxReq req) {
        cabinetBoxService.CabinetBoxExist(req.getId());
        return inventoryMapper.getSumByCabinetBox(req.getId());
    }

    //通过组织机构id将该组织下的装备进行导出
    @Override
    public void ExportInventoryExcel(InventoryReq req){
        //判断组织机构是否存在
        pubOrgService.PubOrgExist(req.getOrgId());
        //查询该组织机构下的所有装备信息
        List<InventoryVo> list = inventoryMapper.selectListByOrg(req.getOrgId());
        HttpServletResponse response = HttpServletUtil.getResponse();
        ExcelExportParam param = new ExcelExportParam();
        param.setDataList(list);
        param.setClazz(Inventory.class);
        param.setResponse(response);
        param.setFileName("装备列表.xls");
        //对数据进行导出
        officeExcelApi.easyExportDownload(param);
    }

    //获取装备的出入库信息
    @Override
    public List<InOutRecordsDto> getInOutRecords(InventoryReq req) {
        //通过装备id获取出入库记录（判断id是否存在
        InventoryExist(req.getId());
        List<LogDetail> list = logDetailService.list(new LambdaQueryWrapper<LogDetail>()
                .eq(LogDetail::getInventoryId, req.getId()));
        List<InOutRecordsDto> inOutRecordsDtoList=new ArrayList<>();
        for (LogDetail logDetail: list) {
            //获取
            LogSummary logSummary = logSummaryService.getById(logDetail.getSummaryId());
            InOutRecordsDto inOutRecordsDto = new InOutRecordsDto();
            BeanPlusUtil.copyProperties(logSummary,inOutRecordsDto);
            inOutRecordsDtoList.add(inOutRecordsDto);
        }
        return inOutRecordsDtoList;
    }

    //查询装备数量报表
    @Override
    public List<EqsSumDto> GetListEquipment(InventoryReq req) {
        //通过获取传递的查询条件进行查询，得到最终的装备数量报表
        return inventoryMapper.GetListEquipment(req);
    }

    //通过id判断装备是否存在
    @Override
    public Inventory InventoryExist(String id) {
        Inventory invMap=this.getById(id);
        if (ObjectUtil.isNull(invMap)){
            throw new ServiceException(InventoryExceptionEnum.INVENTORY_NOT_EXIST);
        }
        return invMap;
    }

}