package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.junmp.jyzb.api.bean.dto.OrderDetailDto;
import com.junmp.jyzb.api.bean.dto.OrderDto;
import com.junmp.jyzb.api.bean.dto.OrderMainDto;
import com.junmp.jyzb.api.bean.query.OrderDetailReq;
import com.junmp.jyzb.api.bean.query.OrderMainReq;
import com.junmp.jyzb.api.bean.req.DetailListReq;
import com.junmp.jyzb.api.bean.req.UpdateOrderReq;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.api.exception.enums.OrderExceptionEnum;
import com.junmp.jyzb.entity.*;
import com.junmp.jyzb.mapper.OrderMainMapper;
import com.junmp.jyzb.service.OrderDetailService;
import com.junmp.jyzb.service.OrderService;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.core.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.fasterxml.jackson.databind.ObjectMapper;

import javax.annotation.Resource;
import java.util.*;


@Service
public class OrderServiceImpl  extends ServiceImpl<OrderMainMapper, OrderMain> implements OrderService {

    @Resource
    private OrderDetailService orderDetailService;
    @Resource
    private OrderMainMapper orderMainMapper;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Resource
    private RabbitAdmin rabbitAdmin;
    @Resource
    private ObjectMapper objectMapper;




    @Transactional(rollbackFor = Exception.class)
    @Override
    public String AddOrder(UpdateOrderReq req) {
        OrderMain order = new OrderMain();
        BeanPlusUtil.copyProperties(req, order);
        //保存
        this.save(order);
        List<OrderDetail> detailList = new ArrayList<>();
        //通过遍历批量保存详细信息
        req.getDetailList().forEach(p->
        {
            OrderDetail detail=new OrderDetail();
            BeanPlusUtil.copyProperties(p, detail);
            detail.setOrderId(order.getId());
            //设置单据类型
            detail.setType(order.getOrderType());
            detailList.add(detail);
        });
        orderDetailService.saveBatch(detailList);
        return order.getId();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public String AddFinishOrder(UpdateOrderReq req) {
        OrderMain order = new OrderMain();
        BeanPlusUtil.copyProperties(req, order);
        //保存
        this.save(order);
        List<OrderDetail> detailList = new ArrayList<>();
        //通过遍历批量保存详细信息
        req.getDetailList().forEach(p->
        {
            OrderDetail detail=new OrderDetail();
            BeanPlusUtil.copyProperties(p, detail);
            detail.setOrderId(order.getId());
            //设置单据类型
            detail.setType(order.getOrderType());
            detailList.add(detail);
        });
        orderDetailService.saveBatch(detailList);
        //将完成的任务单直接推送到消息队列rabbitmq中（需要判断是入库单还是出库单，入库单传发物单位id，出库单传收物单位id）
        //方法1：一个交换机，一个队列。通过中间对象，object存储对象，type表标识
//        if(req.getOrderType().equals("in")){
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getStartOrgId()));
//        }else {
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getEndOrgId()));
//        }
        //方法2：一个交换机，多个队列。动态创建队列
        String exchangeName="123456";
        //需要判断是入库单还是出库单，入库单传发物单位id，出库单传收物单位id
        if(req.getOrderType().equals("in")){
            Queue queue=new Queue(req.getStartOrgId(),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getStartOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getStartOrgId(), req);

        }else {
            Queue queue=new Queue(req.getEndOrgId(),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getEndOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getEndOrgId(), req);

        }

        return order.getId();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean updateOrder(UpdateOrderReq req) {
        OrderMain order = orderMainExist(req.getId());
        if (!order.getExamineState().equals("none"))
        {
            throw new ServiceException(OrderExceptionEnum.ORDER_CAN_NOT_UPDATE);
        }
        List<OrderDetail> list= orderDetailService.QueryList(req.getId());
        Set<String> idSet =new HashSet<>();
        for (OrderDetail orderDetail:list){
            idSet.add(Long.toString(orderDetail.getId()));
        }
        //将该任务单的所有id转换为String,并添加到updateList中，用来判断删除或者修改新增的是否存在
        //判断删除列表是否为空，如果不为空则进行批量删除
        if (req.getDeleteList() != null && !req.getDeleteList().isEmpty())
        {
            //判断传递的list Id是否存在，如果不存在则抛出异常
            for (String s:req.getDeleteList()){
                if (idSet.contains(s)){
                    orderDetailService.removeBatchByIds(req.getDeleteList());
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }
            }
        }
        //对detaillist进行处理
        List<OrderDetail> addList = new ArrayList<>();
        List<OrderDetail> updateList = new ArrayList<>();
        for (DetailListReq detailListReq :req.getDetailList()) {
            //判断是否存在，存在则进行修改
            if (idSet.contains(detailListReq.getId())){
                OrderDetail one = orderDetailService.getOne(new LambdaQueryWrapper<OrderDetail>()
                        .eq(OrderDetail::getId, detailListReq.getId()));
                BeanPlusUtil.copyProperties(detailListReq,one);
                one.setType(req.getOrderType());
                updateList.add(one);
            }else {
                //否则判断id是否为空或者错误，如果为空则添加，不为空则抛出异常
                if (detailListReq.getId()==null || detailListReq.getId().equals("")){
                    OrderDetail detail=new OrderDetail();
                    BeanPlusUtil.copyProperties(detailListReq, detail);
                    detail.setOrderId(req.getId());
                    detail.setType(req.getOrderType());
                    addList.add(detail);
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }

            }
        }
        orderDetailService.updateBatchById(updateList);
        orderDetailService.saveBatch(addList);
        BeanPlusUtil.copyProperties(req,order);
        this.updateById(order);
        return  true;
    }
    public boolean hasCommonElements(List<?> list1, List<?> list2) {
        for (Object item1 : list1) {
            if (list2.contains(item1)) {
                return true; // 找到相同值
            }
        }
        return false; // 没有相同值
    }


    //通过任务单id获取任务单信息orderMain
    @Override
    public OrderMainDto showOrderMain(OrderMainReq req) {
        //判断任务单是否存在
        OrderMain orderMain = orderMainExist(req.getId());
        OrderMainDto orderMainDto = new OrderMainDto();
        BeanPlusUtil.copyProperties(orderMain,orderMainDto);
        return orderMainDto;
    }


    //根据任务单id查询详细信息
    @Override
    public List<OrderDetailDto> getOrderDetail(OrderDetailReq req) {
        orderMainExist(req.getOrderId());
        List<OrderDetail> list = orderDetailService.QueryList(req.getOrderId());
        List<OrderDetailDto> orderDetailDtoList=new ArrayList<>();
        for (OrderDetail orderDetail:list) {
            OrderDetailDto orderDetailDto = new OrderDetailDto();
            BeanPlusUtil.copyProperties(orderDetail,orderDetailDto);
            orderDetailDtoList.add(orderDetailDto);
        }
        return orderDetailDtoList;
    }

    //获取审核状态为finished的任务单，将其推送到rabbitmq中
    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<OrderDto> getOrder() {
        List<OrderDto> orderDtoList=new ArrayList<>();
        //创建方法直接查询到所有审核状态为finished的对应的任务单的id并且是没有被上传到rabbitmq中的单据
        List<OrderMain> finishedList = orderMainMapper.getList();
        List<OrderDetailDto> orderDetailDtoList=new ArrayList<>();
        finishedList.forEach(f->{
            OrderDto orderDto = new OrderDto();
            BeanPlusUtil.copyProperties(f,orderDto);
            List<OrderDetail> list = orderDetailService.list(new LambdaQueryWrapper<OrderDetail>()
                    .eq(OrderDetail::getOrderId, f.getId()));
            list.forEach(l->{
                OrderDetailDto orderDetailDto = new OrderDetailDto();
                BeanPlusUtil.copyProperties(l,orderDetailDto);
                orderDetailDtoList.add(orderDetailDto);
            });
            orderDto.setDetailList(orderDetailDtoList);
            orderDtoList.add(orderDto);
        });
        return orderDtoList;
    }

    //判断任务单是否存在
    @Override
    public OrderMain orderMainExist(String id) {
        OrderMain order = this.getById(id);
        if (ObjectUtil.isNull(order)) {
            throw new ServiceException(OrderExceptionEnum.ORDER_NOT_EXIST);
        }
        return order;
    }


}
